/* NessusJ
 * Copyright (C) 1998 Alexis de Bernis
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * This class display the preferences window.
 *
 *  Please run class NessusJ instead.
 */

import java.awt.*;
import java.io.*;
import java.util.*;
import com.sun.java.swing.*;
import com.sun.java.swing.border.*;
import java.awt.event.*;



class Preferences extends JFrame implements WindowListener,ActionListener
{
    DataInputStream fromPrefFile;
    JPanel portScan, portScan1, portScan2, portScan3;
    JRadioButton rangeFrom, rangePrecise, rangeNone;
    JTextField from, to, precise;
    ButtonGroup rangeType;
    Hashtable prefs,pluginSet;
    String name;
    JButton close;
    JPanel button;
    JTabbedPane mainPane;

    Preferences(String name)
    {
        super("Preferences");
        addWindowListener(this);

        this.name=name;

        try
        {
            fromPrefFile=new DataInputStream(new FileInputStream(name));
        }
        catch(FileNotFoundException e)
        {
            System.out.println("Error: file : "+name+" not found");
            return;
        }

        prefs=new Hashtable();
        pluginSet=new Hashtable();

        loadPrefs();

        getContentPane().setLayout(new BorderLayout());

        portScan=new JPanel(new GridLayout(3,0));
        button=new JPanel(new FlowLayout());
        mainPane=new JTabbedPane();

        portScan1=new JPanel(new FlowLayout(FlowLayout.LEFT));
        portScan2=new JPanel(new FlowLayout(FlowLayout.LEFT));
        portScan3=new JPanel(new FlowLayout(FlowLayout.LEFT));
        rangeType=new ButtonGroup();
        rangeFrom=new JRadioButton(" From : ",false);
        rangeFrom.setActionCommand("rangeFrom");
        rangeFrom.addActionListener(this);
        rangeType.add(rangeFrom);
        rangePrecise=new JRadioButton(" Precisely : ",false);
        rangePrecise.setActionCommand("rangePrecise");
        rangePrecise.addActionListener(this);
        rangeType.add(rangePrecise);
        rangeNone=new JRadioButton(" None",true);
        rangeNone.setActionCommand("rangeNone");
        rangeNone.addActionListener(this);
        rangeType.add(rangeNone);
        from=new JTextField(7);
        to=new JTextField(7);
        precise=new JTextField(20);

        portScan1.add(rangeFrom);
        portScan1.add(from);
        portScan1.add(new JLabel(" to : "));
        portScan1.add(to);
        portScan2.add(rangePrecise);
        portScan2.add(precise);
        portScan3.add(rangeNone);
        portScan.add(portScan1);
        portScan.add(portScan2);
        portScan.add(portScan3);
        portScan.setBorder(new TitledBorder("Port Scanning"));

        close=new JButton("Close");
        close.addActionListener(this);
        button.add(close);


        mainPane.add("Port Scanning",portScan);
        getContentPane().add("Center",mainPane);
        getContentPane().add("South",button);

        String temp=(String) prefs.get("scan_range");

        if(temp.indexOf(";")!=-1)
        {
            precise.setText(temp);
            rangePrecise.setSelected(true);
            from.setEnabled(false);
            to.setEnabled(false);

        } else
        {
            from.setText(temp.substring(0,temp.indexOf("-")));
            to.setText(temp.substring(temp.indexOf("-")+1));
            rangeFrom.setSelected(true);
            precise.setEnabled(false);
        }

        temp=null;
        temp=(String) prefs.get("scan_hosts");

        if(temp.equals("no"))
        {
            rangeNone.setSelected(true);
            to.setEnabled(false);
            from.setEnabled(false);
            precise.setEnabled(false);
        }


        pack();

    }


/** This method determines wheter a plugin is activated in the plugin set.
    If it doesn't exist in the prefs file, it will create a new entry in it.
    By default, new plugins are activated.
  */
    public boolean isPluginActivated(String name)
    {
        String temp=(String) pluginSet.get(name);
        if(temp!=null)
            return(toBoolean(temp));

        pluginSet.put(name,"yes");
        return(true);
    }


/** This method saves in the prefs the state of a plugin.
  */
    public void setPluginActivated(String name, boolean temp)
    {
        pluginSet.put(name,toString(temp));
    }

    private boolean toBoolean(String temp)
    {
        if(temp.equalsIgnoreCase("yes"))
            return(true);

        return(false);
    }

    private String toString(boolean temp)
    {
        if(temp==true)
            return("yes");

        return("no");
    }



    private void loadPrefs()
    {
        String read;

        try
        {

            while(fromPrefFile.available()!=0)
            {
                read=fromPrefFile.readLine();

                if(read.equals(""))
                    continue;

                if(read.substring(0,1).equals("#"))
                    continue;
            
                if(read.indexOf(" = ")==-1)
                {
                    if(read.indexOf("begin(")==-1)
                        continue;

                    while(!((read=fromPrefFile.readLine()).equals("end")))
                        pluginSet.put(read.substring(0,read.indexOf(" = ")),
                                      read.substring(read.indexOf(" = ")+3));

                    continue;
                }
                                  
                prefs.put(read.substring(0,read.indexOf(" = ")),
                          read.substring(read.indexOf(" = ")+3));
            }
        
        } catch(IOException e)
        {
            System.out.println("Error reading prefs file");
        }
    }



    private void writePrefs()
    {
        DataOutputStream toPrefsFile;
        Enumeration keys, elements;

        try
        {
            toPrefsFile=new DataOutputStream(new FileOutputStream(name));

            toPrefsFile.writeBytes("# This file was automagically created by NessusJ\n");

            keys=prefs.keys();
            elements=prefs.elements();

            while(keys.hasMoreElements())
                toPrefsFile.writeBytes(keys.nextElement()+" = "+elements.nextElement()+"\n");

            toPrefsFile.writeBytes("begin(plugin_set)\n");

            keys=null;
            elements=null;
            keys=pluginSet.keys();
            elements=pluginSet.elements();

            while(keys.hasMoreElements())
                toPrefsFile.writeBytes(keys.nextElement()+" = "+
                                       elements.nextElement()+"\n");
            
            toPrefsFile.writeBytes("end\n");
        } catch(IOException e)
        {
            System.out.println("Error writing prefs file");
            return;
        }
    }



    protected void finalize()
    {
        writePrefs();
        try
        {
            super.finalize();
        }catch(Throwable e)
        {
            return;
        }
    }

    public String get(String key)
    {
        return((String) prefs.get(key));
    }

    public void actionPerformed(ActionEvent evt)
    {
        String command=evt.getActionCommand();

        if(command.equals("Close"))
        {
            if(rangeFrom.isSelected())
                prefs.put("scan_range",from.getText()+"-"+to.getText());

            if(rangePrecise.isSelected())
                prefs.put("scan_range",precise.getText());

            setVisible(false);
        }

        if(command.equals("rangeFrom"))
        {
            prefs.put("scan_hosts","yes");
            from.setEnabled(true);
            to.setEnabled(true);
            precise.setEnabled(false);
        }

        if(command.equals("rangePrecise"))
        {
            prefs.put("scan_hosts","yes");
            from.setEnabled(false);
            to.setEnabled(false);
            precise.setEnabled(true);
        }

        if(command.equals("rangeNone"))
        {
            prefs.put("scan_hosts","no");
            from.setEnabled(false);
            to.setEnabled(false);
            precise.setEnabled(false);
        }


    }

    public void windowClosing(WindowEvent evt)
    {
        if(rangeFrom.isSelected())
            prefs.put("scan_range",from.getText()+"-"+to.getText());

        if(rangePrecise.isSelected())
            prefs.put("scan_range",precise.getText());

        setVisible(false);
    }        

    public void windowClosed(WindowEvent evt ) { }
    public void windowOpened(WindowEvent evt) { }
    public void windowOpening(WindowEvent evt ) { }
    public void windowIconified(WindowEvent evt ) { }
    public void windowActivated(WindowEvent evt ) { }
    public void windowDeiconified(WindowEvent evt ) { }
    public void windowDeactivated(WindowEvent evt ) { }


}
